//  *****************************************************************************
//   						main.c
// 
//              Demonstracny program pre dosku KTK ARM7
//
//              Ovladanie jasu LED diody na zaklade potenciometra pomocou SDM modulacie
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"

// Priorita prerusenia PIT
#define PIT_INTERRUPT_LEVEL 0       // 0 - najnizsia priorita prerusenia

// Konstanty pre AD prevodnik
#define   TRGEN    (0x0)    // Softverove spustanie prevodu
#define   TRGSEL   (0x0)    // nema vplyv pri pouziti softveroveho spustania
#define   LOWRES   (0x1)    // 8-bitovy prevod
#define   SLEEP    (0x0)    // normalny mod
#define   PRESCAL  (0x9)    // ADC hodiny = MCK / ((PRESCAL+1)*2)
#define   STARTUP  (0x7)    // Cas startu prevodnika = (STARTUP+1)*8 / ADC hodiny
#define   SHTIM    (0x3)    // Cas Sample And Hold = (SHTIM+1) / ADC hodiny

// Globalne premenne
int intenzita=0;     // intenzita svietenia LED (0-63)
int vystup=0;        // vystupna intenzita


//  *******************************************************
//  Obsluha prerusenia od periodickeho casovaca PIT
//  *******************************************************
void PIT_prerusenie(void)
{ int pom;

  // vymazanie ziadosti o prerusenie
  AT91C_BASE_PITC->PITC_PIVR;

  // maskovanie hornych bitov intenzity (necha sa len 6 spodnych)
  pom = vystup & 0x3f;

  // vypocet novej intezity
  vystup = intenzita + pom;

  // uprava vystupu podla najvyssieho bitu (siedmeho)
  if (vystup & 0x40)
    AT91C_BASE_PIOA->PIO_SODR = D1;         // zapnutie LED
  else
    AT91C_BASE_PIOA->PIO_CODR = D1;         // vypnutie LED  
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main(void) 
{ int nap;

  //******************* Uvodna inicializacia *******************
  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);
	
  //******************* Inicializacia portu A ******************
  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
	
  // konfiguracia jednotlivych pinov
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = D1;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = D1;

  // vsetko je vypnute
  AT91C_BASE_PIOA->PIO_CODR = D1;

  //******************* Nastavenie prerusenia od PIT ******************
  // Povolenie hodin pre PIT
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_SYS;
  // nastavenie periody casovaca na 0.1 ms
  // do PIT vstupuje frekvencia MCK/16
  AT91C_BASE_PITC->PITC_PIMR = (int)((MCK/16)*0.0001)-1;
  // povolenie casovaca a prerusenia
  AT91C_BASE_PITC->PITC_PIMR = AT91C_BASE_PITC->PITC_PIMR | AT91C_PITC_PITEN | AT91C_PITC_PITIEN;

  // nastavenie obsluzneho programu
  AT91C_BASE_AIC->AIC_SVR[AT91C_ID_SYS] = (unsigned int) PIT_prerusenie;
  // zdroj prerusenia a priorita
  AT91C_BASE_AIC->AIC_SMR[AT91C_ID_SYS] = PIT_INTERRUPT_LEVEL;
  // zrusenie prerusenia (ak nejake nahodou bolo)
  AT91C_BASE_AIC->AIC_ICCR = 0x1 << AT91C_ID_SYS;
  // povolenie prerusenia od PIT
  AT91C_BASE_AIC->AIC_IECR = 0x1 << AT91C_ID_SYS;

  //******************* Nastavenie prevodnika ******************
  // Povolenie hodin pre ADC
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_ADC;
  // Nastavenie pracovneho modu prevodnika  
  AT91C_BASE_ADC->ADC_MR = (SHTIM << 24) | (STARTUP << 16) | 
                           (PRESCAL << 8) | (SLEEP << 5) | 
                           (LOWRES << 4) | (TRGSEL << 1) | TRGEN;
  // vyber aktivnych kanalov - aktivny je len kanal 0
  AT91C_BASE_ADC->ADC_CHER = 1;
  
  //******************* Hlavna cast programu *******************
  // nekonecna slucka
  while(1)
  { // start prevodu
    AT91C_BASE_ADC->ADC_CR = AT91C_ADC_START;
    // cakanie na ukoncenie prevodu
    while (!(AT91C_BASE_ADC->ADC_SR & 0x1));
    // nacitanie vysledku prevodu - kanal 0
    nap = AT91C_BASE_ADC->ADC_CDR0;           // nap je v rozmedzi 0-255
    // vypocet intenzity svietenia
    intenzita = nap >> 2;  // nap >> 2 je to iste ako nap/4, intenzita je v rozmedzi 0-63
  }
}
